@@ -23,10 +23,14 @@
 https://foundationdb.com/documentation/api-python.html"""
 
 def open(*args, **kwargs):
-    raise RuntimeError('You must call api_version(100) before using any fdb methods')
+    raise RuntimeError('You must call api_version(200) before using any fdb methods')
 
 init = open
 
+def _add_symbols(module, symbols):
+    for symbol in symbols:
+        globals()[symbol] = getattr(module, symbol)
+
 def api_version(ver):
     if '_version' in globals():
         if globals()['_version'] != ver:
@@ -36,13 +40,15 @@ def api_version(ver):
     if ver < 13:
         raise RuntimeError('FDB API versions before 13 are not supported')
 
-    if ver > 100:
-        raise RuntimeError('Latest known FDB API version is 100')
+    if ver > 200:
+        raise RuntimeError('Latest known FDB API version is 200')
 
     import fdb.impl
 
-    err = fdb.impl._capi.fdb_select_api_version_impl(ver, 100)
+    err = fdb.impl._capi.fdb_select_api_version_impl(ver, 200)
     if err != 0:
+        if err == 2203: # api_version_not_supported, but that's not helpful to the user
+            raise RuntimeError("API version %d not supported by the installed FoundationDB C library" % ver)
         raise RuntimeError('FoundationDB API error')
 
     list = (
@@ -61,8 +67,7 @@ def api_version(ver):
         'StreamingMode',
         )
 
-    for sym in list:
-        globals()[sym] = getattr(fdb.impl, sym)
+    _add_symbols(fdb.impl, list)
 
     if ver > 22:
         import fdb.locality
@@ -91,3 +96,12 @@ def api_version(ver):
         setattr(fdb.impl.FDBRange, "next", next)
 
     globals()['_version'] = ver
+
+    import fdb.directory_impl
+    directory_symbols = ('directory', 'DirectoryLayer',)
+    _add_symbols(fdb.directory_impl, directory_symbols)
+
+    import fdb.subspace_impl
+    subspace_symbols = ('Subspace',)
+    _add_symbols(fdb.subspace_impl, subspace_symbols)
+
