@@ -23,8 +23,9 @@ import random
 import struct
 
 from fdb import impl as _impl
+from fdb import six
 import fdb.tuple
-from subspace_impl import Subspace
+from .subspace_impl import Subspace
     
 class HighContentionAllocator (object):
 
@@ -45,7 +46,7 @@ class HighContentionAllocator (object):
         window = self._window_size(start)
         if (count + 1) * 2 >= window:
             # Advance the window
-            del tr[self.counters: self.counters[start].key() + chr(0)]
+            del tr[self.counters: self.counters[start].key() + six.int2byte(0)]
             start += window
             del tr[self.recent: self.recent[start]]
             window = self._window_size(start)
@@ -60,7 +61,7 @@ class HighContentionAllocator (object):
             # subsequent risk of conflict for this transaction.
             candidate = random.randrange(start, start + window)
             if tr[self.recent[candidate]] == None:
-                tr[self.recent[candidate]] = ""
+                tr[self.recent[candidate]] = b'' 
                 return fdb.tuple.pack((candidate,))
 
     def _window_size(self, start):
@@ -74,7 +75,7 @@ class HighContentionAllocator (object):
         return 8192
 
 class Directory(object):
-    def __init__(self, directory_layer, path=(), layer=''):
+    def __init__(self, directory_layer, path=(), layer=b''):
         self._directory_layer = directory_layer
         self._path = path
         self._layer = layer
@@ -156,7 +157,7 @@ class Directory(object):
 
 class DirectoryLayer(Directory):
 
-    def __init__(self, node_subspace=Subspace(rawPrefix="\xfe"), content_subspace=Subspace(), allow_manual_prefixes=False):
+    def __init__(self, node_subspace=Subspace(rawPrefix=b'\xfe'), content_subspace=Subspace(), allow_manual_prefixes=False):
         Directory.__init__(self, self)
 
         # If specified, new automatically allocated prefixes will all fall within content_subspace
@@ -238,7 +239,7 @@ class DirectoryLayer(Directory):
         node = self._node_with_prefix(prefix)
         tr[parent_node[self.SUBDIRS][path[-1]]] = prefix
         if not layer:
-            layer = ''
+            layer = b''
 
         tr[node[b'layer']] = layer
         
@@ -405,7 +406,7 @@ class DirectoryLayer(Directory):
 
             return
 
-        version = struct.unpack('<III', str(version))
+        version = struct.unpack('<III', bytes(version))
 
         if version[0] > self.VERSION[0]:
             raise Exception("Cannot load directory with version %d.%d.%d using directory layer %d.%d.%d" % (version + self.VERSION))
@@ -423,7 +424,7 @@ class DirectoryLayer(Directory):
         if key.startswith(self._node_subspace.key()):
             return self._root_node
         for k, v in tr.get_range(self._node_subspace.range(()).start,
-                                 self._node_subspace.pack((key,)) + "\x00",
+                                 self._node_subspace.pack((key,)) + b'\x00',
                                  reverse=True,
                                  limit=1):
             prev_prefix = self._node_subspace.unpack(k)[0]
@@ -438,7 +439,7 @@ class DirectoryLayer(Directory):
     def _contents_of_node(self, node, path, layer=None):
         prefix = self._node_subspace.unpack(node.key())[0]
 
-        if layer == 'partition':
+        if layer == b'partition':
             return DirectoryPartition(self._path + path, prefix, self)
         else:
             return DirectorySubspace(self._path + path, prefix, self, layer)
@@ -447,7 +448,7 @@ class DirectoryLayer(Directory):
         n = _Node(self._root_node, (), path)
         for i, name in enumerate(path):
             n = _Node(self._node_with_prefix(tr[n.subspace[self.SUBDIRS][name]]), path[:i+1], path)
-            if not n.exists() or n.layer(tr) == 'partition':
+            if not n.exists() or n.layer(tr) == b'partition':
                 return n
         return n
 
@@ -476,18 +477,18 @@ class DirectoryLayer(Directory):
         return len(list(tr.get_range(prefix, _impl.strinc(prefix), limit=1))) == 0
 
 def _to_unicode_path(path):
-    if isinstance(path, str):
-        path = unicode(path)
+    if isinstance(path, bytes):
+        path = six.text_type(path)
 
-    if isinstance(path, unicode):
+    if isinstance(path, six.text_type):
         return (path,)
 
     if isinstance(path, tuple):
         path = list(path)
         for i, name in enumerate(path):
-            if isinstance(name, str):
-                path[i] = unicode(path[i])
-            elif not isinstance(name, unicode):
+            if isinstance(name, bytes):
+                path[i] = six.text_type(path[i])
+            elif not isinstance(name, six.text_type):
                 raise ValueError('Invalid path: must be a unicode string or a tuple of unicode strings')
 
         return tuple(path)
@@ -510,9 +511,9 @@ class DirectorySubspace(Subspace, Directory):
 
 class DirectoryPartition(DirectorySubspace):
     def __init__(self, path, prefix, parent_directory_layer):
-        directory_layer = DirectoryLayer(Subspace(rawPrefix=prefix + '\xfe'), Subspace(rawPrefix=prefix))
+        directory_layer = DirectoryLayer(Subspace(rawPrefix=prefix + b'\xfe'), Subspace(rawPrefix=prefix))
         directory_layer._path = path
-        DirectorySubspace.__init__(self, path, prefix, directory_layer, 'partition')
+        DirectorySubspace.__init__(self, path, prefix, directory_layer, b'partition')
 
         self._parent_directory_layer = parent_directory_layer
 
@@ -575,7 +576,7 @@ class _Node (object):
         return self._layer
 
     def is_in_partition(self, tr=None, include_empty_subpath=False):
-        return self.exists() and self.layer(tr) == 'partition' and (include_empty_subpath or len(self.target_path) > len(self.path))
+        return self.exists() and self.layer(tr) == b'partition' and (include_empty_subpath or len(self.target_path) > len(self.path))
 
     def get_partition_subpath(self):
         return self.target_path[len(self.path):]
