@@ -29,6 +29,8 @@ import platform
 import os
 import sys
 
+from fdb import six
+
 _network_thread = None
 _network_thread_reentrant_lock = threading.RLock()
 
@@ -68,6 +70,16 @@ def option_wrap_string(code):
         self._parent._set_option(code, param, length)
     return setfunc
 
+def option_wrap_bytes(code):
+    def setfunc(self, param=None):
+        if param is None:
+            self._parent._set_option(code, None, 0)
+        elif isinstance(param, bytes):
+            self._parent._set_option(code, param, len(param))
+        else:
+            raise TypeError('Value must be of type ' + bytes.__name__)
+    return setfunc
+
 def option_wrap_int(code):
     def setfunc(self, param):
         self._parent._set_option(code, struct.pack("<q", param), 8)
@@ -88,6 +100,8 @@ def fill_options(scope):
             f = option_wrap(code)
         elif paramType == type(''):
             f = option_wrap_string(code)
+        elif paramType == type(b''): # This won't happen in Python 2 because type('') == type(b''), but it will happen in Python 3
+            f = option_wrap_bytes(code)
         elif paramType == type(0):
             f = option_wrap_int(code)
         else:
@@ -327,7 +341,7 @@ class TransactionRead(_FDBBase):
         self._snapshot = snapshot
 
     def __del__(self):
-        # print 'Destroying transactionread 0x%x' % self.tpointer
+        # print('Destroying transactionread 0x%x' % self.tpointer)
         self.capi.fdb_transaction_destroy(self.tpointer)
 
     def get_read_version(self):
@@ -491,12 +505,12 @@ class Future(_FDBBase):
     Event = threading.Event
 
     def __init__(self, fpointer):
-        # print 'Creating future 0x%x' % fpointer
+        # print('Creating future 0x%x' % fpointer)
         self.fpointer = fpointer
 
     def __del__(self):
         if self.fpointer:
-            # print 'Destroying future 0x%x' % self.fpointer
+            # print('Destroying future 0x%x' % self.fpointer)
             self.capi.fdb_future_destroy(self.fpointer)
             self.fpointer = None
 
@@ -620,13 +634,17 @@ class LazyFuture(Future):
         return self.value
 
 
+# This is a workaround to avoid a compiler issue as described here:
+# http://bugs.python.org/issue12370
+_super = super
+
 class FutureString(LazyFuture):
     def __init__(self, *args):
         self._error = None
-        super(FutureString, self).__init__(*args)
+        _super(FutureString, self).__init__(*args)
 
     def getclass(self):
-        return str
+        return bytes
     __class__ = property(getclass)
 
     def as_foundationdb_key(self):
@@ -638,6 +656,9 @@ class FutureString(LazyFuture):
     def __str__(self):
         return self.value.__str__()
 
+    def __bytes__(self):
+        return self.value
+
     def __repr__(self):
         return self.value.__repr__()
 
@@ -752,7 +773,7 @@ class Database(FormerFuture):
         self.options = _DatabaseOptions(self)
 
     def __del__(self):
-        # print 'Destroying database 0x%x' % self.dpointer
+        # print('Destroying database 0x%x' % self.dpointer)
         self.capi.fdb_database_destroy(self.dpointer)
 
     @staticmethod
@@ -876,7 +897,7 @@ class Cluster(FormerFuture):
         self.options = _ClusterOptions(self)
 
     def __del__(self):
-        # print 'Destroying cluster 0x%x' % self.cpointer
+        # print('Destroying cluster 0x%x' % self.cpointer)
         self.capi.fdb_cluster_destroy(self.cpointer)
 
     def open_database(self, name):
@@ -948,6 +969,9 @@ class KVIter(object):
         else:
             raise StopIteration
 
+    def __next__(self):
+        return self.next()
+
 
 class KeyValueStruct(ctypes.Structure):
     _fields_ = [('key', ctypes.POINTER(ctypes.c_byte)),
@@ -1229,7 +1253,7 @@ def init(event_model=None):
             class NetworkThread(threading.Thread):
                 def run(self):
                     _capi.fdb_run_network()
-                    # print 'Network stopped'
+                    # print('Network stopped')
 
             _network_thread = NetworkThread()
             _network_thread.daemon = True
@@ -1358,4 +1382,4 @@ def strinc(key):
     if len(key) == 0:
         raise ValueError('Key must contain at least one byte not equal to 0xFF.')
 
-    return key[:-1] + chr(ord(key[-1]) + 1)
+    return key[:-1] + six.int2byte(ord(key[-1:]) + 1)
